@@ -6,8 +6,11 @@ melt <- function(data, ..., na.rm = FALSE, value.name = "value") {
       reshape2::melt(data, ..., na.rm=na.rm, value.name=value.name)
 }
 
-patterns <- function(...) {
+patterns <- function(..., cols=character(0)) {
     p = unlist(list(...), use.names=FALSE)
+    if (!is.character(p))
+        stop("Input patterns must be of type character.")
+    lapply(p, grep, cols)
 }
 
 melt.data.table <- function(data, id.vars, measure.vars, variable.name = "variable", 
@@ -18,22 +21,28 @@ melt.data.table <- function(data, id.vars, measure.vars, variable.name = "variab
     if (missing(measure.vars)) measure.vars = NULL
     measure.sub = substitute(measure.vars)
     if (is.call(measure.sub) && measure.sub[[1L]] == "patterns") {
-        measure.vars = lapply(eval(measure.sub), grep, names(data))    
+        measure.sub = as.list(measure.sub)[-1L]
+        idx = which(names(measure.sub) %in% "cols")
+        if (length(idx)) {
+            cols = eval(measure.sub[["cols"]], parent.frame())
+            measure.sub = measure.sub[-idx]
+        } else cols = names(data)
+        pats = lapply(measure.sub, eval, parent.frame())
+        measure.vars = patterns(pats, cols=cols)
     }
-    if (is.list(measure.vars)) {
+    if (is.list(measure.vars) && length(measure.vars) > 1L) {
         if (length(value.name) == 1L)  
           value.name = paste(value.name, seq_along(measure.vars), sep="")
     }
-    ans <- .Call("Cfmelt", data, id.vars, measure.vars, 
+    ans <- .Call(Cfmelt, data, id.vars, measure.vars, 
             as.logical(variable.factor), as.logical(value.factor), 
             variable.name, value.name, as.logical(na.rm), 
             as.logical(verbose))
     setDT(ans)
     if (any(duplicated(names(ans)))) {
-        cat("Duplicate column names found in molten data.table. Setting unique names using 'make.names'")   
+        cat("Duplicate column names found in molten data.table. Setting unique names using 'make.names'\n")
         setnames(ans, make.unique(names(ans)))
     }
     setattr(ans, 'sorted', NULL)
     ans
 }
-
